# Информация о API V4L2  
## Доступные контролы
Данные контролы доступны через стандартный интерфейс (API) V4L2 и могут быть изменены с помощью стандартных v4l2-ctl и qv4l2.
В заголовочном файле `/usr/src/forward-*/include/forward-v4l2-ioctl.h` имеются все необходимые объявления для использования этих контролов в своём коде.
Большинство контролов доступно как для входов, так и для выходов. Также, большинство из них можно менять во время приема/передачи потока, без остановки.
Некоторые контролы являются общими для всей платы - при изменении в одном устройстве они меняются для всех устройств.

* `V4L2_CID_FORWARD_ENABLE_VANC` (`bool`) - включить захват / проигрывание VANC-области **только при отключеном проигрывании**
* `V4L2_CID_FORWARD_WIDESCREEN` (`bool`) - входной сигнал имеет флаг widescreen в VPID / поставить флаг widescreen на выходе
* `V4L2_CID_FORWARD_HW_TIMESTAMP` (`u32`) - получить аппаратное время. На платах имеется счетчик, который работает на частоте 148.5MHz, синхронно с выходами - аппартные часы. **только чтение**
* `V4L2_CID_FORWARD_FREQUENCY_ADJUST` (`u64`) - подстройка выходной частоты в ppb (point-per-trillion, 10^-12). Записывая значения в этот контрол можно плавно (без потери сигнала на выходе) поменять несущую частоту на выходе - `Fout = V4L2_CID_FORWARD_FREQUENCY_ADJUST/1e-12 * Fbase`. Не рекомендуется менять значение больше чем на 10ppm (1e-6) за один раз - это может привести к потере сигнала на входах и выходах на несколько секунд. Предел регулирования - ±500ppm **общий для всей платы**
* `V4L2_CID_FORWARD_ASI_PERIOD` (`enum`) - переключить период приема/выдачи данных (буферов) из драйвера для ASI **только при отключеном проигрывании**
    - `20ms (0)` - *по-умолчанию*
    - `10ms (1)`
    - `5ms (2)`
    - `2.5ms (3)`
* `V4L2_CID_FORWARD_TIMECODE` (`enum`) - вид таймкода для приема на входе/вид таймкода который нужно выдать.
    - `None (0)` - не принимать/не выдавть таймкод - *по-умолчанию*
    - `LTC (1)` - принимать только LTC / выдавать LTC
    - `VITC (2)` - принимать только VITC / выдавать VITC
    - `Any (3)` - принимать любой таймкод / невалидно для выхода
* `V4L2_CID_FORWARD_GENLOCK_SOURCE` (`enum`) - источник генлока, привязка несущей частоты. **общий для всей платы**
    - `V4L_FORWARD_GENLOCK_SRC_MASTER (0)` - мастер, несущая частота не зависит от входов, её можно изменить с помощью `V4L2_CID_FORWARD_FREQUENCY_ADJUST`, *по-умолчанию*
    - `V4L_FORWARD_GENLOCK_SRC_ANALOG (1)` - привязать несущую частоту к аналоговому входу (black-burst/tri-level-sync)
    - `V4L_FORWARD_GENLOCK_SRC_IN0 (2)` - привязать несущую частоту к первому входу SDI
    - `V4L_FORWARD_GENLOCK_SRC_IN1 (3)` - привязать несущую частоту ко второму входу SDI
* `V4L2_CID_FORWARD_GENLOCK_ENABLE` (`bool`) - привязать положение кадра (фазу) к источнику генлока. Без этого флага кадры будут идти с частотой источника генлока, но их начало будет неопределенным. **только для выхода**
* `V4L2_CID_FORWARD_GENLOCK_STATE` (`enum`) - текущее состояние генлока **только чтение** **общий для всей платы**
    - `V4L_FORWARD_GENLOCK_MASTER (0)` - в режиме мастер 
    - `V4L_FORWARD_GENLOCK_NO_INPUT_SIGNAL (1)` - входной сигнал-источник генлока не обнаружен
    - `V4L_FORWARD_GENLOCK_LOCKING (2)` - идёт подстройка частоты и фазы
    - `V4L_FORWARD_GENLOCK_LOCKED (3)` - выхода в генлоке, всё хорошо
    - `V4L_FORWARD_GENLOCK_HOLDOVER (3)` - плата была в генлоке, а потом пропал сигнал, плата пытается удержать частоту
* `V4L2_CID_FORWARD_GENLOCK_OFFSET` (`u32`) - сдвиг выходного кадра относительно источника генлока. Единица - `6.734ns` = `1/148.5MHz` = 1 пиксель формата 1080p50/p60  **только для выхода**

## Доступные события
Драйвер посылает события через стандартный интерфейс (API) V4L2 - https://linuxtv.org/downloads/v4l-dvb-apis/userspace-api/v4l/dev-event.html
В заголовочном файле `/usr/src/forward-*/include/forward-v4l2-ioctl.h` имеются все необходимые объявления для использования этих событий в своём коде.
Большинство контролов доступно как для входов, так и для выходов.

Драйвер посылает следующие события:

* `V4L2_EVENT_SOURCE_CHANGE` - стандартное событие смены формата
* `V4L2_EVENT_VSYNC` - стандартное событие синхронизации, посылается раз в поле, вне зависимости от того, включено проигрывание или нет.
* `V4L2_EVENT_FRAME_SYNC` - стандартное событие синхронизации, посылается раз в кадр перед тем, как отдать буфер в приложение при включенном проигрывании. Содержит текущий номер кадра.
* `V4L2_EVENT_FORWARD_TIMESTAMP` - специфичное для драйвера событие, содержит текущий таймштамп кадра, посылается раз в поле, вне зависимости от того, включено проигрывание или нет. В событии передается структура `struct v4l2_event_forward_timestamp` со следующими полями:
    - `buffer_sequence` - текущий номер кадра, совпадает с буфером и `V4L2_EVENT_FRAME_SYNC`
    - `field` - текущее поле, `V4L2_FIELD_*`
    - `eof_timestamp` - аппартный таймштамп поля/кадра, формат тот же самый, что и в `V4L2_CID_FORWARD_HW_TIMESTAMP`
    - `irq_timestamp` - аппартный таймштамп прерывания - время когда информация дошла до драйвера, формат тот же самый, что и в `V4L2_CID_FORWARD_HW_TIMESTAMP`
