#include "Utils.hpp"

#include <iomanip>

std::ostream& operator<<(std::ostream& str, const v4l2_dv_timings& tim)
{
    int width = tim.bt.width;
    int height = tim.bt.height;
    int fps100 = (uint64_t)tim.bt.pixelclock * 100 / V4L2_DV_BT_FRAME_WIDTH(&tim.bt)
        / V4L2_DV_BT_FRAME_HEIGHT(&tim.bt);
    bool interlaced = tim.bt.interlaced ? true : false;
    if (interlaced)
        fps100 *= 2;

    // Modes with /1.001 framerate are designated by V4L2_DV_FL_REDUCED_FPS flag
    if (tim.bt.flags & V4L2_DV_FL_REDUCED_FPS)
        fps100 = fps100 * 1000 / 1001;

    str << width << "x" << height << (interlaced ? "i" : "p") << fps100 / 100;
    if ((fps100 % 100) != 0)
        str << "." << std::setw(2) << std::setfill('0') << (fps100 % 100);

    return str;
}

std::ostream& operator<<(std::ostream& str, const v4l2_format& fmt)
{
    if ((fmt.type == V4L2_BUF_TYPE_VIDEO_CAPTURE) || (fmt.type == V4L2_BUF_TYPE_VIDEO_OUTPUT)) {
        v4l2_pix_format pix = fmt.fmt.pix;
        char fourcc[5] = { '0', '0', '0', '0' };
        for (int i = 0; i < 4; i++)
            fourcc[i] = (pix.pixelformat >> (i * 8)) & 0xFF;

        str << "(" << pix.width << "x" << pix.height;
        switch (pix.field) {
        case V4L2_FIELD_INTERLACED:
            str << ", interlaced";
            break;
        case V4L2_FIELD_ALTERNATE:
            str << ", alternate";
            break;
        case V4L2_FIELD_NONE:
            str << ", progressive";
            break;
        }

        str << ", " << fourcc << ", " << pix.sizeimage << ")";
    } else if ((fmt.type == V4L2_BUF_TYPE_VBI_CAPTURE) || (fmt.type == V4L2_BUF_TYPE_VBI_OUTPUT)) {
        v4l2_vbi_format vbi = fmt.fmt.vbi;
        char fourcc[5] = { '0', '0', '0', '0' };
        for (int i = 0; i < 4; i++)
            fourcc[i] = (vbi.sample_format >> (i * 8)) & 0xFF;

        str << "(" << vbi.sampling_rate << "Hz, " << vbi.start[0] << "+" << vbi.count[0] << ", "
            << vbi.start[1] << "+" << vbi.count[1] << ", " << vbi.samples_per_line << "spl"
            << ")";
    }
    return str;
}

void generateRP219ColorBarUYVY(uint8_t* buf, int width, int height)
{
    int a = width, b = height;
    double c = a * 3.0 / 4.0 / 7.0;
    double d = a / 4.0 / 2.0;

    for (int y = 0; y < height; y++) {
        for (int x = 0; x < width; x += 2) {
            uint32_t* out = (uint32_t*)&buf[(y * width + x) * 2];
            if (x < d) {
                *out = 0x68806880;
            } else if (x < d + 1 * c) {
                *out = 0xB480B480;
            } else if (x < d + 2 * c) {
                *out = 0xA888A82C;
            } else if (x < d + 3 * c) {
                *out = 0x912C9193;
            } else if (x < d + 4 * c) {
                *out = 0x8534853F;
            } else if (x < d + 5 * c) {
                *out = 0x3FCC3FC1;
            } else if (x < d + 6 * c) {
                *out = 0x33D4336D;
            } else if (x < d + 7 * c) {
                *out = 0x1D781DD4;
            } else {
                *out = 0x68806880;
            }
        }
    }
}
